package Catalyst::Authentication::Credential::SangerSSO;
# Author:        david.jackson@sanger.ac.uk
# Maintainer:    $Author: ajb $
# Created:       2010-04-28
# Last Modified: $Date: 2011-08-10 10:05:48 +0100 (Wed, 10 Aug 2011) $
# Id:            $Id: SangerSSO.pm 13877 2011-08-10 09:05:48Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/useful_modules/tags/release-38.0/lib/Catalyst/Authentication/Credential/SangerSSO.pm $

use strict;
use warnings;
use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 13877 $ =~ /(\d+)/mxs; $r; };
use UNIVERSAL::require;

sub authenticate : method {
    my ( $self, $c, $realm) = @_;
    $c->log->debug('SangerSSO authenticate() called from ' . $c->request->uri) ;

    local $ENV{'CONTENT_LENGTH'}=0; #to avoid hangs experienced when intial call to server is a POST
    my $sangerweb = SangerWeb->require?SangerWeb->new():undef;
    $c->log->debug('SangerWeb object '.($sangerweb?q():'not ').' created in authenticate');

    if ( $sangerweb and my $username = $sangerweb->username() ) {
      $c->log->debug("Obtained $username from SangerWeb");
      return $realm->find_user({username=>$username}, $c);
    }
    $c->log->debug($sangerweb?'No username from SangerWeb':'No SangerWeb object');
    return;
}

1;
__END__

=head1 NAME

        Catalyst::Authentication::Credential::SangerSSO

=head1 VERSION

$LastChangedRevision: 13877 $

=head1 SYNOPSIS

    #action in a controller
    sub login : Local {
        my ( $self, $c ) = @_;
        my $user     = $c->req->params->{user};
        my $realm = $c->req->params->{realm};
        if (defined $user or defined $realm) {
            my $password     = $c->req->params->{password};
            $c->logout;
            if ( $c->authenticate( defined $user ? { username => $user, password => $password } :{} , $realm) ) {
                # login correct
            } else {
                # login incorrect
            }
        }
    }

 #config
 <Plugin::Authentication>
  default_realm = dbic_realm
  <realms>
  <test_realm>
    <credential>
          class = Password
          password_field = password
          password_type = clear
    </credential>
    <store>
          class = Minimal
          <users>
             <bob>
                  password = s00p3r
                  editor = yes
                  roles = edit
                  roles = delete
             </bob>
             <dj3>
                  password = s3cr3t
                  roles = comment
                  roles = foofoo
             </dj3>
          </users>
    </store>
  </test_realm>
  <ssso_mini_realm>
    <credential>
                        class           = SangerSSO
    </credential>
    <store>
          class = Minimal
          <users>
             <dj3>
                  roles = admin
                  roles = superfoo
             </dj3>
          </users>
    </store>
  </ssso_mini_realm>
  <mini_realm>
    <credential>
                        class           = Testing
                        password_field  = password
                        global_password = secret
    </credential>
    <store>
          class = Minimal
          <users>
             <dj3>
                  roles = admin
                  roles = rant
             </dj3>
          </users>
    </store>
  </mini_realm>
  <dbic_realm>
    <credential>
                        class           = Testing
                        password_field  = password
                        global_password = secret
    </credential>
    <store>
          class = DBIx::Class
          user_model = tracking::User
          role_relation = usergroups
          role_field = groupname
          use_userdata_from_session = 1
    </store>
  </dbic_realm>
  <ssso_realm>
    <credential>
                        class           = SangerSSO
    </credential>
    <store>
          class = DBIx::Class
          user_model = tracking::User
          role_relation = usergroups
          role_field = groupname
          use_userdata_from_session = 1
    </store>
  </ssso_realm>
  </realms>
 </Plugin::Authentication>


=head1 DESCRIPTION

Catalyst Credential module to allow authentication using the Sanger's web single sign on system.

=head1 SUBROUTINES/METHODS

=head2 authenticate

Called by Catalyst authentication infrastructure....

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Catalyst::Authentication

=item Readonly

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by David K Jackson (david.jackson@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
