#############
# $Id: test_type_of_value.pm 13877 2011-08-10 09:05:48Z ajb $
# Created By: ajb
# Mast Maintained By: $Author: ajb $
# Created On: 2020-06-16
# Last Changed On: $Date: 2011-08-10 10:05:48 +0100 (Wed, 10 Aug 2011) $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/useful_modules/tags/release-38.0/lib/npg_common/roles/test_type_of_value.pm $

package npg_common::roles::test_type_of_value;
use Moose::Role;

use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 13877 $ =~ /(\d+)/mxs; $r; };
## no critic (RequirePodAtEnd)

=head1 NAME

npg_common::roles::test_type_of_value

=head1 VERSION

$LastChangedRevision: 13877 $

=head1 SYNOPSIS

  package My::Package;
  use Moose;
  with qw{npg_common::roles::test_type_of_value};

=head1 DESCRIPTION

This role imports methods which allow you to test the type of value stored in a variable

i.e. Object, HashRef, ArrayRef...

It uses the standard types of Moose Attributes to test against. All methods take a variable as input, and return a boolean true/false

Exceptions:

Any, Item - You will presumably have something in the variable
Bool - The variable will have a true or false definition, just test if ( $variable ) {}
Maybe[`a], Undef, Defined - You should be testing definedness using standard Perl idioms

That a ScalarRef/ArrayRef/HashRef contain a type - Loop through and test them yourself, or set up a deliberate Attribute to test this

ClassName/RoleName: You need to have the class/role already imported. Deliberately decided to leave this out.

=head1 SUBROUTINES/METHODS

=head2 is_array
=head2 is_code
=head2 is_filehandle
=head2 is_glob
=head2 is_hash
=head2 is_int
=head2 is_num
=head2 is_object
=head2 is_regexp
=head2 is_scalar (Value)
=head2 is_scalar_ref
=head2 is_string

=cut

sub is_array {
  my ( $self, $variable ) = @_;
  my $is_array = 0;
  eval {
    $self->_i_am_an_array( $variable ); # test if this is an array
    $is_array++;
  } or do {};
  return $is_array;
}

has q{_i_am_an_array} => (
  isa => q{ArrayRef},
  is => q{rw},
);



sub is_code {
  my ( $self, $variable ) = @_;
  my $is_code = 0;
  eval {
    $self->_i_am_a_code( $variable ); # test if this is an code
    $is_code++;
  } or do {};
  return $is_code;
}
has q{_i_am_a_code} => (
  isa => q{CodeRef},
  is => q{rw},
);


sub is_filehandle {
  my ( $self, $variable ) = @_;
  my $is_filehandle = 0;
  eval {
    $self->_i_am_a_filehandle( $variable ); # test if this is a filehandle
    $is_filehandle++;
  } or do {};
  return $is_filehandle;
}
has q{_i_am_a_filehandle} => (
  isa => q{FileHandle},
  is => q{rw},
);


sub is_glob {
  my ( $self, $variable ) = @_;
  my $is_glob = 0;
  eval {
    $self->_i_am_a_glob( $variable ); # test if this is a glob
    $is_glob++;
  } or do {};
  return $is_glob;
}
has q{_i_am_a_glob} => (
  isa => q{GlobRef},
  is => q{rw},
);


sub is_hash {
  my ( $self, $variable ) = @_;
  my $is_hash = 0;
  eval {
    $self->_i_am_a_hash( $variable ); # test if this is an hash
    $is_hash++;
  } or do {};
  return $is_hash;
}

has q{_i_am_a_hash} => (
  isa => q{HashRef},
  is => q{rw},
);


sub is_int {
  my ( $self, $variable ) = @_;
  my $is_int = 0;
  eval {
    $self->_i_am_an_int( $variable ); # test if this is an int
    $is_int++;
  } or do {};
  return $is_int;
}

has q{_i_am_an_int} => (
  isa => q{Int},
  is => q{rw},
);


sub is_num {
  my ( $self, $variable ) = @_;
  my $is_num = 0;
  eval {
    $self->_i_am_a_num( $variable ); # test if this is a num
    $is_num++;
  } or do {};
  return $is_num;
}
has q{_i_am_a_num} => (
  isa => q{Num},
  is => q{rw},
);


sub is_object {
  my ( $self, $variable ) = @_;
  my $is_object = 0;
  eval {
    $self->_i_am_an_object( $variable ); # test if this is an object
    $is_object++;
  } or do {};
  return $is_object;
}

has q{_i_am_an_object} => (
  isa => q{Object},
  is => q{rw},
);


sub is_regexp {
  my ( $self, $variable ) = @_;
  my $is_regex = 0;
  eval {
    $self->_i_am_a_regexp( $variable ); # test if this is a regex
    $is_regex++;
  } or do {};
  return $is_regex;
}
has q{_i_am_a_regexp} => (
  isa => q{RegexpRef},
  is => q{rw},
);


sub is_scalar {
  my ( $self, $variable ) = @_;
  my $is_scalar = 0;
  eval {
    $self->_i_am_a_scalar( $variable ); # test if this is an scalar
    $is_scalar++;
  } or do {};
  return $is_scalar;
}

has q{_i_am_a_scalar} => (
  isa => q{Value},
  is => q{rw},
);


sub is_scalar_ref {
  my ( $self, $variable ) = @_;
  my $is_scalar_ref = 0;
  eval {
    $self->_i_am_a_scalar_ref( $variable ); # test if this is a scalar_ref
    $is_scalar_ref++;
  } or do {};
  return $is_scalar_ref;
}

has q{_i_am_a_scalar_ref} => (
  isa => q{ScalarRef},
  is => q{rw},
);


sub is_string {
  my ( $self, $variable ) = @_;
  my $is_string = 0;
  eval {
    $self->_i_am_a_string( $variable ); # test if this is a string
    $is_string++;
  } or do {};
  return $is_string;
}

has q{_i_am_a_string} => (
  isa => q{Str},
  is => q{rw},
);


1;
__END__

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose::Role

=item Carp

=item English -no_match_vars

=item Readonly

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

$Author: ajb $

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Andy Brown (ajb@sanger.ac.uk)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
