#########
# Author:        gq1
# Maintainer:    $Author: ajb $
# Created:       2010-03-01
# Last Modified: $Date: 2011-08-10 10:05:48 +0100 (Wed, 10 Aug 2011) $
# Id:            $Id: Fastq2SAM.pm 13877 2011-08-10 09:05:48Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/useful_modules/tags/release-38.0/lib/npg_common/sequence/Fastq2SAM.pm $
#

package npg_common::sequence::Fastq2SAM;

use strict;
use warnings;
use Moose;
use Carp;
use English qw(-no_match_vars);
use Readonly;

with 'MooseX::Getopt';

use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 13877 $ =~ /(\d+)/mxs; $r; };

# Bitwise flags
Readonly::Scalar my $PAIRED_READ_FLAG   =>    1;
Readonly::Scalar my $MAPPED_PAIR_FLAG   =>    2;
Readonly::Scalar my $UNMAPPED_READ_FLAG =>    4;
Readonly::Scalar my $UNMAPPED_MATE_FLAG =>    8;
#Readonly::Scalar my $READ_STRAND_FLAG   =>   16;
#Readonly::Scalar my $MATE_STRAND_FLAG   =>   32;
Readonly::Scalar my $IS_READ_1_FLAG     =>   64;
Readonly::Scalar my $IS_READ_2_FLAG     =>  128;
#Readonly::Scalar my $NOT_1RY_ALIGN_FLAG =>  256;
#Readonly::Scalar my $PLATFORM_FAIL_FLAG =>  512;
#Readonly::Scalar my $IS_DUPLICATE_FLAG  => 1024;

# Blank fields for the SAM file.
Readonly::Scalar my $RNAME => q{*};
Readonly::Scalar my $POS   => 0;
Readonly::Scalar my $MAPQ  => 0;
Readonly::Scalar my $CIGAR => q{*};
Readonly::Scalar my $MRNM  => q{*};
Readonly::Scalar my $MPOS  => q{0};
Readonly::Scalar my $ISIZE => q{0};

## no critic (Documentation::RequirePodAtEnd)

=head1 NAME

npg_common::sequence::Fastq2SAM

=head1 VERSION

$LastChangedRevision: 13877 $

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 fastq1

fastq1 

=cut

has 'fastq1'       => (isa            => 'Str',
                       is             => 'rw',
                       required       => 1,
                       documentation  => 'first end fastq file',
                      );

=head2 fastq2

fastq2 

=cut

has 'fastq2'       => (isa            => 'Str',
                       is             => 'rw',
                       documentation  => 'second end fastq file',
                      );

=head2 process

read each fastq files at the same time and print out as sam

=cut

sub process {
  my $self = shift;

  my $fastq1 = $self->fastq1();
  my $fastq2 = $self->fastq2();
  my $flag1 = $self->_get_flag(1);
  my $flag2;

  open my $fastq_fh1, '<', $fastq1 or croak qq{can not open $fastq1};## no critic (InputOutput::RequireBriefOpen)
  my $fastq_fh2;
  if($fastq2){
    open $fastq_fh2, '<', $fastq2 or croak qq{can not open $fastq2};## no critic (InputOutput::RequireBriefOpen)
    $flag2 = $self->_get_flag(2);
  }

  while( my $sam_line1 = $self->_get_one_read_from_fastq($fastq_fh1, $flag1) ){
    print $sam_line1, "\n" or croak "can not print $sam_line1";

    if($fastq2){
      my $sam_line2 = $self->_get_one_read_from_fastq($fastq_fh2, $flag2);
      print $sam_line2, "\n" or croak "can not print $sam_line2";
    }
  }

  close $fastq_fh1 or croak qq{can not close $fastq1};
  if($fastq2){
    close $fastq_fh2 or croak qq{can not close $fastq2};
  }
  return 1;
}

sub _get_one_read_from_fastq{
  my ($self, $fastq_fh, $flag) = @_;

  my $qname = <$fastq_fh>;

  if(!$qname){
    return;
  }
  chomp $qname;
  #croak 'fasq file format wong' if $qname !~ /^[@]/msx;    # In case of broken fastq files.
  $qname =~ s{^@}{}msx;
  $qname =~ s{/[12]\s*}{}msx;

  my $seq = <$fastq_fh>;
  chomp $seq;

  my $qheader = <$fastq_fh>;

  my $qual = <$fastq_fh>;
  chomp $qual;

  my $sam_line = join "\t", ( $qname, $flag, $RNAME, $POS,   $MAPQ,
                                    $CIGAR, $MRNM, $MPOS,  $ISIZE, $seq,
                                    $qual );

  return $sam_line;
}

sub _get_flag{
  my ($self, $end) = @_;

  if($end == 2 && !$self->fastq2()){
    croak 'No second end fastq file given, you can get the flag for sam';
  }

  my $flag = 0;
  if($self->fastq2()){
    $flag = 1;
  }

  my $switch = $end * $IS_READ_1_FLAG;

  $flag |= $UNMAPPED_READ_FLAG;
  if($self->fastq2()){
     $flag |= $switch;
     $flag |= $UNMAPPED_MATE_FLAG;
  }
  return $flag;
}

no Moose;

1;
__END__

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose

=item MooseX::Getopt

=item Carp

=item English -no_match_vars

=item Readonly

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Guoying Qi E<lt>gq1@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2009 GRL, by Guoying Qi

This file is part of NPG.

NPG is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

=cut
