#########
# Author:        gq1
# Maintainer:    $Author: ajb $
# Created:       2010-03-01
# Last Modified: $Date: 2011-08-10 10:05:48 +0100 (Wed, 10 Aug 2011) $
# Id:            $Id: SplitFastqByTag.pm 13877 2011-08-10 09:05:48Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/useful_modules/tags/release-38.0/lib/npg_common/sequence/SplitFastqByTag.pm $
#

package npg_common::sequence::SplitFastqByTag;

use strict;
use warnings;
use Moose;
use Carp;
use English qw(-no_match_vars);
use File::Basename;
use autodie qw(:all);

use IO::Tee;

with 'MooseX::Getopt';

our $VERSION   = do { my ($r) = q$Revision: 13877 $ =~ /(\d+)/mxs; $r; };


## no critic (Documentation::RequirePodAtEnd)

=head1 NAME

npg_common::sequence::SplitFastqByTag

=head1 VERSION

$LastChangedRevision: 13877 $

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 SUBROUTINES/METHODS

=head2 fastq

fastq file to be split 

=cut

has 'fastq'        => (isa            => 'Str',
                       is             => 'rw',
                       required       => 1,
                       documentation  => 'fastq file name to be split',
                      );

=head2 output_dir

output directory

=cut

has 'output_dir'   => (isa            => 'Str',
                       is             => 'rw',
                       required       => 1,
                       documentation  => 'output directory',
                      );

=head2 limit

Maximum number of entries to write to the tag files.

=cut

has 'limit'        => (isa            => 'Int',
                       is             => 'rw',
                       predicate      => 'has_limit',
                       documentation  => 'How many sequences to write',
                      );

=head2 process

split the given fastq by tag

=cut
sub process {
  my $self = shift;

  my $fastq = $self->fastq();
  if(!-e $fastq){
    croak "Given fastq file $fastq not exisits!";
  }

  my $output_dir = $self->output_dir();
  if (! -e $output_dir ) {
    mkdir $output_dir;
  }

  my ($fastq_name, $dir, $suffix) = fileparse( $fastq, qw(.fastq) );

  my %fh_list = ();
  my %count   = ();

  ## no critic (InputOutput::RequireBriefOpen)
  $self->has_limit() && ( open my $null_fh, q{>}, '/dev/null' );

  open my $fastq_fh, q{<}, $fastq;

  while(my $seq_name = <$fastq_fh>){

    chomp $seq_name;

    my ($tag) = $seq_name =~ m/^@.*[#](\d+)/mxs;

    if ( !defined $tag ) {
      croak "Fastq format wrong: $fastq";
    }
    my $fh_by_tag = $fh_list{$tag};

    if ( $self->has_limit() ) {
       ( defined $count{$tag} ) || ( $count{$tag} = 0 );
       if ( $count{$tag} >= $self->limit() ) {
         $fh_by_tag = $null_fh;
       }
    }

    if(! $fh_by_tag ){
       my $fastq_by_tag_name = qq{$output_dir/$fastq_name}.qq{#$tag.fastq};
       my $fastqcheck_by_tag_name = $fastq_by_tag_name . q{check};
       my $fastq_md5_by_tag_name = $fastq_by_tag_name . q{.md5};
       ## no critic (RequireBriefOpen ProhibitTwoArgOpen InputOutput::RequireBriefOpen)
       open my $fh_fastq, '>', $fastq_by_tag_name;
       open my $fh_fastqcheck, qq{ | fastqcheck > $fastqcheck_by_tag_name }
                               or croak "Failed to pipe to $fastqcheck_by_tag_name:\n$ERRNO";
       open my $fh_fastq_md5, qq{ | md5sum > $fastq_md5_by_tag_name}
                               or croak "Failed to calcualte md5 to $fastq_md5_by_tag_name:\n$ERRNO";
       ## use critic
       $fh_by_tag = IO::Tee->new($fh_fastq,$fh_fastqcheck, $fh_fastq_md5);
       $fh_list{$tag} =  $fh_by_tag;
    }
    print {$fh_by_tag} "$seq_name\n" or croak "Cannot print on file by tag $tag";
    my $seq = <$fastq_fh>;
    print {$fh_by_tag} $seq or croak "Cannot print on file by tag $tag";
    my $seq_name2 = <$fastq_fh>;
    print {$fh_by_tag} $seq_name2 or croak "Cannot print on file by tag $tag";
    my $qual = <$fastq_fh>;
    print {$fh_by_tag} $qual or croak "Cannot print on file by tag $tag";
    $count{$tag}++;
  }

  close $fastq_fh;
  $self->has_limit() && ( close $null_fh );

  foreach my $tag (keys %fh_list){
    my $fh_by_tag = $fh_list{$tag};
    if($fh_by_tag){
      $fh_by_tag->close() or  croak $ERRNO;
    }
  }

  return 1;
}

no Moose;

1;
__END__

=head1 DIAGNOSTICS

=head1 CONFIGURATION AND ENVIRONMENT

=head1 DEPENDENCIES

=over

=item Moose

=item MooseX::Getopt

=item Carp

=item English -no_match_vars

=item File::Basename

=back

=head1 INCOMPATIBILITIES

=head1 BUGS AND LIMITATIONS

=head1 AUTHOR

Guoying Qi E<lt>gq1@sanger.ac.ukE<gt>

=head1 LICENSE AND COPYRIGHT

Copyright (C) 2010 GRL, by Guoying Qi

This file is part of NPG.

NPG is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

=cut
