#!/software/bin/perl -w

#########
# Author:        gq1
# Maintainer:    $Author: ajb $
# Created:       2009-05-13
# Last Modified: $Date: 2011-08-10 10:05:48 +0100 (Wed, 10 Aug 2011) $
# Id:            $Id: fastq_srf_split.pl 13877 2011-08-10 09:05:48Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/useful_modules/tags/release-38.0/scripts/fastq_srf_split.pl $
#

use strict;
use warnings;
use lib qw(lib /software/solexa/lib/perl5/);
use Carp;
use English qw{-no_match_vars};
use Getopt::Long;
use npg_common::Fastq_Split;
use File::Basename;

use Readonly; Readonly::Scalar our $VERSION => do { my ($r) = q$LastChangedRevision: 13877 $ =~ /(\d+)/mx; $r; };

main();
0;

sub main {
  my $opts = initialise();

  my $srf_filter_cmd = '/software/solexa/bin/srf_filter';
  my $fastq_split_command = '/software/solexa/bin/fastq_split.pl';
  #my $fastq_split_command = '/nfs/users/nfs_g/gq1/dev/new-pipeline-dev/useful_modules/branches/current/scripts/fastq_split.pl';
  my $xsl_filename = '/software/solexa/src/new-pipeline-dev/useful_modules/trunk/data/xsl/splitting_stats.xsl';
  
  if(!$opts || $opts->{help} || scalar keys %{$opts} < 2 )
  {
  print q[Usage:
  fastq_srf_split

  Options:
    -fastq1     filename1
    -fastq2     filename2
    -fastqt     filename_t
    -srf        srf_filename
    -output_dir output_dir
    -ref_bwa    ref_bwa_prefix
    -ref_fa     ref_fa
    -ref_name   reference_name
    -help
    -fork_alignment
    -id_run     run_id
    -position   position
    -stats_only
    -no_pileup

    At least one fastq file needs to be provided by fastq1 and fastq2.
    output_dir must be given.
    Optional fastqt for tags fastq file to be split.
    Optional srf file to be split.
    Optional reference location in BWA format, default human will be used if not given.
    Optional Fasta format with samtools fai, default human will be used if not given.
    Optional reference name, default human will be used if not given.
    Optional id_run and position.
] or croak qq{Error printing help: $EVAL_ERROR};
  exit 2;
  }
  
  print  {*STDERR} qq{Spliting fastq and srf files...\n};
  my ($sec,$min,$hour,$day,$month,$yr19,@rest) =   localtime(time);
  printf {*STDERR} qq{Time:\t%02d:%02d:%02d\n}, $hour, $min, $sec;
  
  if($opts->{fastq1}){

    $fastq_split_command .= q{ -fastq1 }.$opts->{fastq1};
  }

  if($opts->{fastq2}){

    $fastq_split_command .= q{ -fastq2 }.$opts->{fastq2};
  }

  if($opts->{fastqt}){

    $fastq_split_command .= q{ -fastqt }.$opts->{fastqt};
  }

  $fastq_split_command .= q{ -output_dir }.$opts->{output_dir};

  if($opts->{ref_fa}){

    $fastq_split_command .= q{ -ref_fa }.$opts->{ref_fa};
  }
  if($opts->{ref_bwa}){

    $fastq_split_command .= q{ -ref_bwa }.$opts->{ref_bwa};
  }

  my $ref_name = q{human};
  if($opts->{ref_name}){

    $fastq_split_command .= q{ -ref_name }.$opts->{ref_name};
    $ref_name = $opts->{ref_name};
  }
  
  if($opts->{fork_alignment}){
  
    $fastq_split_command .= q{ -fork_alignment};
  }

  if($opts->{id_run}){
  
    $fastq_split_command .= q{ -id_run }.$opts->{id_run};
  }
  
  if($opts->{position}){
  
    $fastq_split_command .= q{ -position }.$opts->{position};
  }

  if($opts->{stats_only}){
  
    $fastq_split_command .= q{ -stats_only };
  }
  
  if($opts->{no_pileup}){
  
    $fastq_split_command .= q{ -no_pileup };
  }

  my $srf_filename = $opts->{srf};
  if($srf_filename) {
    $fastq_split_command .= q{ -srf_filter};
  }

  print {*STDERR} "$fastq_split_command\n";

  my $fastq_split_rs = system $fastq_split_command;
  if($fastq_split_rs != 0){

    croak "failed to split fastq files";
  }

  my $output_dir = $opts->{output_dir};
  
  my $fastq1_name = $opts->{fastq1};
  my $fastq2_name = $opts->{fastq2};

  $fastq1_name = npg_common::Fastq_Split->get_filename_no_extension($fastq1_name) if ($fastq1_name);
  $fastq2_name = npg_common::Fastq_Split->get_filename_no_extension($fastq2_name) if ($fastq2_name);

  my $both_fastq;
  if($fastq1_name && $fastq2_name){

    $both_fastq = $fastq1_name.q{-}.$fastq2_name;
  }elsif($fastq1_name){

    $both_fastq = $fastq1_name;
  }elsif($fastq2_name){

    $both_fastq = $fastq2_name;
  }

  my $filter_file_hit = $output_dir.qq{/$both_fastq}.qq{_$ref_name.filter};
  my $filter_file_non_hit =$output_dir.qq{/$both_fastq}.qq{_non$ref_name.filter};

  my $cp_xsl_cmd = "cp $xsl_filename $output_dir/qc/splitting_stats.xsl";
  my $cp_xsl = system $cp_xsl_cmd;
  if($cp_xsl != 0){
    print {*STDERR} "failed: $cp_xsl_cmd\n";
  }

  if($srf_filename){

    if( (-e $srf_filename ) && (-e $filter_file_hit ) && (-e $filter_file_non_hit)){

      my $fastq_split = npg_common::Fastq_Split->new({
         srf_filter_cmd  => $srf_filter_cmd,
      });

      my $args_srf_split = {srf_filename        => $srf_filename,
                            filter_file_hit     => $filter_file_hit,
                            filter_file_non_hit => $filter_file_non_hit,
                            output_dir          => $output_dir,
                            ref_name            => $ref_name,
               };
      $fastq_split->split_srf_file($args_srf_split);
    }else{

      croak "SRF or SRF filter files no exist";
    }
  }else{

    print {*STDERR} "No SRF file provided\n";
  }
  #delete srf filter files
  unlink $filter_file_hit, $filter_file_non_hit;
  
  ($sec,$min,$hour,$day,$month,$yr19,@rest) =   localtime(time);
  printf {*STDERR}  qq{Time:\t%02d:%02d:%02d\n}, $hour, $min, $sec;
  print {*STDERR} "Finish spliting fastq and srf files\n";

}

# ----------------------------------------------------------------------
sub initialise {

    my %options = ();

    my $rc = GetOptions(\%options,
                        'help',
                        'fastq1:s',
                        'fastq2:s',
                        'fastqt:s',
                        'srf:s',
                        'id_run:i',
                        'position:i',
                        'output_dir:s',
                        'ref_bwa:s',
                        'ref_fa:s',
                        'ref_name:s',
                        'fork_alignment',
                        'stats_only',
                        'no_pileup',          
                        );
    return \%options;
}
