#########
# Author:        gq1
# Maintainer:    $Author: ajb $
# Created:       2009-04-07
# Last Modified: $Date: 2011-08-10 10:05:48 +0100 (Wed, 10 Aug 2011) $
# Id:            $Id: 10-Fastq_Split.t 13877 2011-08-10 09:05:48Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/useful_modules/tags/release-38.0/t/10-Fastq_Split.t $
#

use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Test::More tests => 13;
use npg_common::Fastq_Split;
use Digest::MD5;
use Perl6::Slurp;
use lib q{t};
use t::util;

my $util = t::util->new();
my $work_dir = $util->temp_directory();

use_ok('npg_common::Fastq_Split');

{
  my $fastq_split = npg_common::Fastq_Split->new(
    {
      srf_filter_cmd => 't/bin/srf_filter',
      samtools_cmd    =>'t/bin/samtools',
    }
  );


#--
 is($fastq_split->get_filename_no_extension('t/data/fastq_split/seq_ids.txt'), q{seq_ids}, 'get correct file name without extension');

#--
  my $ids = $fastq_split->_get_seq_ids_from_file('t/data/fastq_split/seq_ids.txt');
  is(scalar keys %{$ids}, 2, 'correct number ids got');

#--
  eval{
    $fastq_split->_srf_filter_file($ids, "$work_dir/srf_filter.txt");
    1;
  };
  is($EVAL_ERROR, q{}, 'no croak to generate srf filter file');
diag $work_dir, "\n\n";
#-- 
  my $select_fastq = "$work_dir/select.fastq";
  my $select_fastqcheck = "$work_dir/select.fastqcheck";
  my $select_md5 =  "$work_dir/select.fastq.md5";
  my $rest_fastq = "$work_dir/rest.fastq";
  my $rest_fastqcheck = "$work_dir/rest.fastqcheck"; 
  my $rest_md5 = "$work_dir/rest.fastq.md5";


  eval{
    $fastq_split->fastq_split('t/data/fastq_split/2605_1_1.fastq', $ids, $select_fastq, $rest_fastq);
    1;
  };
  is($EVAL_ERROR, q{}, 'no croak to split fastq files');
  my $md5 = Digest::MD5->new();
  $md5->add(slurp $select_fastq);
  is($md5->hexdigest(), q{cda48093db9884ec704db5e1c56df60e}, 'correct select fastq md5');
  $md5->add(slurp $select_fastqcheck);
  is($md5->hexdigest(), q{ff714d7fe860f549df1313c3bfe0b30f}, 'correct select fastq check md5');
  $md5->add(slurp $rest_fastq);
  is($md5->hexdigest(), q{972fc40eb4c621a05fbef631ec6efc22}, 'correct rest fastq md5');
  $md5->add(slurp $rest_fastqcheck);
  is($md5->hexdigest(), q{ec31a5a65b84a21b55ead3335fbf1bf1}, 'correct rest fastq check md5');
  like((slurp $select_md5), qr{cda48093db9884ec704db5e1c56df60e}, 'correct select fastq md5 file');
  like( (slurp $rest_md5), qr{72fc40eb4c621a05fbef631ec6efc22}, 'correct rest fastq md5 file');


#--
  eval{
    $fastq_split->srf_filter('t/data/fastq_split/2605_1.srf', 't/data/fastq_split/srf_filter_ids.txt', "$work_dir/select.srf");
    1;
  };
  is($EVAL_ERROR, q{}, 'no croak in farm for fastqindex split');
}

{
  my $fastq_split = npg_common::Fastq_Split->new(
    {
      bwa_cmd         =>'t/bin/bwa',
      srf_filter_cmd  =>'t/bin/srf_filter',
      samtools_cmd    =>'t/bin/samtools',
    }
  );
  my $args = {
    fastq1     => 't/data/fastq_split/2605_1_1.fastq',
    fastq2     => 't/data/fastq_split/2605_1_1.fastq',
    srf        => 't/data/fastq_split/2605_1.srf',
    ref_bwa    => 't/data/fastq_split/Homo_sapiens.NCBI36.48.dna.all.fa',
    ref_fa     => 't/data/fastq_split/Homo_sapiens.NCBI36.48.dna.all.fa',
    output_dir => $work_dir,
  };
  eval{
    $fastq_split->file_split($args);
    1;
  };
  like($EVAL_ERROR, qr{The bam file to be sorted.+ not exist}, 'croak to split fastq and srf file');
}
