#########
# Author:        gq1
# Maintainer:    $Author: ajb $
# Created:       2009-08-27
# Last Modified: $Date: 2011-08-10 10:05:48 +0100 (Wed, 10 Aug 2011) $
# Id:            $Id: 10-irods-run-Bam.t 13877 2011-08-10 09:05:48Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/useful_modules/tags/release-38.0/t/10-irods-run-Bam.t $
#

use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Test::More tests => 49;
use Test::Exception;

local $ENV{'NPG_WEBSERVICE_CACHE_DIR'} = q[t/data/sequence/];

use_ok('npg_common::irods::run::Bam');

{
  my $bam = npg_common::irods::run::Bam->new(id_run => 5174, archive_path=>'t/data/sequence/100818_IL32_05174/Latest_Summary/archive');

  is( $bam-> _bam_file_name({id_run=>5174, position=>1}), '5174_1.bam', 'lane 1 bam file name');

  is( $bam-> _bam_file_name({id_run=>5174, position=>1, human_split=>'nonhuman', tag_index=>0}), '5174_1_nonhuman#0.bam', 'lane 1 tag_index 0 nonhuman bam file name');

  my $npg_api_run = $bam->npg_api_run();
  isa_ok ( $npg_api_run, 'npg::api::run');
  is($npg_api_run->id_run(), 5174, 'correct id_run 5174');
  ok($npg_api_run->is_multiplexed, '5174 is multiplex run');
  
  is( scalar @{$bam->file_list()}, 6, "correct number of bam files");
 
  my $names_1 = $bam->get_study_library_sample_names(5174, 1);
  
  ok ( !$names_1->{is_phix_control}, 'run 5174 lane 1 is not phix control data');

  my $study_names_1 = $names_1->{study};
  is( scalar @{$study_names_1}, 1, 'only 1 study name for run 5174 lane 1' );
  is( $study_names_1->[0], 'ZF_MrSol4000', 'study name for run 5174 lane 1');  
  is($names_1->{study_title}->[0], "Zebrafish Tilling by Illumina Sequencing", 'correct study title'); 
  is($names_1->{study_accession_number}->[0], "ERP000110", 'correct study accession nubmer');

  my $sample_names_1 = $names_1->{sample};
  is( scalar @{$sample_names_1}, 2, '2 sample names for run 5174 lane 1' );
  is( $sample_names_1->[0], '83_3_1-11', 'first sample name for run 5174 lane 1');
  

  is($names_1->{sample_public}->[0], 'Zebrafish', 'first sample public name');
  is( scalar @{$names_1->{sample_public}}, 1, '1 sample public name for run 5174 lane 1' );

  is( scalar @{$names_1->{sample_accession_number}}, 2, '2 sample public accession number for run 5174 lane 1' );
  
  is($names_1->{sample_accession_number}->[0], 'ERS009168', 'first sample accession number');
  is($names_1->{sample_accession_number}->[1], 'ERS009169', 'second sample accession number');

  is( scalar @{ $names_1->{library} }, 1, 'one library name for run 5174 lane 1');
  is( $names_1->{library}->[0], 'MrSol4000Pool18', 'library name for run 5174 lane 1' );
  
  is( scalar @{ $names_1->{library_id} }, 1, 'one library id for run 5174 lane 1');
  is( $names_1->{library_id}->[0], '151127', 'library name for run 5174 lane 1' );

  my $names_4 = $bam->get_study_library_sample_names(5174, 4);
  
  ok ( $names_4->{is_phix_control}, 'run 5174 lane 4 is phix control data');

  is( scalar @{ $names_4->{study} }, 0, 'no study name for run 5174 lane 4' );
  is($names_4->{library}->[0], 'Multiplex-phix', 'library name for run 5174 lane 4');
  is($names_4->{sample}->[0], undef, 'sample name for run 5174 lane 4 undefined');
  
  my $names_1_7 = $bam->get_study_library_sample_names(5174, 1, 7);

  ok ( !$names_1_7->{is_phix_control}, 'run 5174 lane 1 plex 7 is not phix control data');

  my $study_names_1_7 = $names_1_7->{study};
  is( scalar @{$study_names_1_7}, 1, 'only 1 study name for run 5174 lane 1 plex 7' );
  is( $study_names_1_7->[0], 'ZF_MrSol4000', 'study name for run 5174 lane 1 plex 7');

  my $sample_names_1_7 = $names_1_7->{sample};
  is( scalar @{$sample_names_1_7}, 1, 'only 1 sample name for run 5174 lane 1 plex 7' );
  is( $sample_names_1_7->[0], '83_3_1-11', 'sample name for run 5174 lane 1 plex 7');
  
  is( $names_1_7->{library}->[0], '83_3_1-11 151125', 'libarary name for run 5174 lane 1 plex 7');

  my $bam_list = {q{5310_1#0.bam} => 1, q{5310_1#1.bam} => 1};
  my $bai_list = {q{5310_1#0.bai} => 1,};
  $bam->number_of_reads_list_irods()->{q{5310_1#0.bam}} = 1000;
  $bam->number_of_reads_list_irods()->{q{5310_1#1.bam}} = 2000;
  ok (!$bam->_check_bam_index_file($bam_list, $bai_list), 'bam index file missing');
  
  $bai_list->{'5310_1#3.bai'} = 1;
  ok (!$bam->_check_bam_index_file($bam_list, $bai_list), 'bam index file missing');

  $bai_list->{'5310_1#1.bai'} = 1;
  $bam_list->{'5310_1#3.bam'} = 1;
  $bam->number_of_reads_list_irods()->{q{5310_1#3.bam}} = 3000;
  ok ($bam->_check_bam_index_file($bam_list, $bai_list), 'bam index file match');

  my $ref1 = '/lustre/scratch101/sanger/references/Homo_sapiens/CGP_GRCh37.NCBI.allchr_MT/all/bwa/Homo_sapiens.GRCh37.NCBI.allchr_MT.fa';
  my $bwa_pg_line1 = q{@}.qq{PG	ID:bwa_aln	PN:bwa aln	CL:/software/solexa/bin/aligners/bwa/bwa-0.5.8c/bwa aln -t 2 $ref1 /nfs/sf20/ILorHSany_sf20/analysis/101101_HS3_05450_A_205NTABXX/Data/Intensities/PB_basecalls_20101108-105238/no_cal/archive/5450_3_1.fastq > /tmp/MUYklDpWoh/1.sai;/software/solexa/bin/aligners/bwa/bwa-0.5.8c/bwa aln  -t 2 $ref1 /nfs/sf20/ILorHSany_sf20/analysis/101101_HS3_05450_A_205NTABXX/Data/Intensities/PB_basecalls_20101108-105238/no_cal/archive/5450_3_2.fastq > /tmp/MUYklDpWoh/2.sai;};

  is($bam->_get_ref_from_bwa_pg($bwa_pg_line1), $ref1, 'correct reference');

  my $ref2 = q{/nfs/repository/d0031/references/Salmonella_enterica/Paratyphi_A_AKU_12601/all/bwa/S_enterica_paratyphi_AKU_12601.fasta};
  my $bwa_pg_line2 = q{@}.qq{PG	ID:bwa	PN:bwa	PP:split_fastq_by_tag	VN:0.5.8c (r1536)	CL:-q 15 -t 2 $ref2};
  is($bam->_get_ref_from_bwa_pg($bwa_pg_line2), $ref2, 'correct reference');

  my $ref3 = '/nfs/srpipe_references/references/Homo_sapiens/GRCh37_53/all/bwa/Homo_sapiens.GRCh37.dna.all.fa';
  my $bwa_pg_line3 = '@PG	ID:bwa_aln	PN:bwa aln	PP:split_fastq_by_tag	VN:0.5.8c (r1536)	DS:bwa alignment from fastq files	CL:/software/solexa/bin/aligners/bwa/bwa-0.5.8c/bwa aln -q 15 -t 2 '. $ref3 . ' /nfs/sf19/ILorHSany_sf19/analysis/110311_HS2_05947_A_B02YHABXX/Data/Intensities/PB_basecalls_20110318-161034/no_cal/archive/lane8/5947_8_1#2.fastq > 1.sai';
  is($bam->_get_ref_from_bwa_pg($bwa_pg_line3), $ref3, 'correct reference');

  open my $sam_fh, '<', 't/data/sequence/5431_6#3.sam';

  is( $bam->_grep_bam_bwa_pg_line ($sam_fh), $bwa_pg_line2, 'correct bwa pg line' );

  close $sam_fh;
  
  is( $bam->get_bam_reference('t/data/sequence/5431_6#3.bam'), $ref2, 'correct reference from bam file');
}

{
  my $bam = npg_common::irods::run::Bam->new(id_run => 5174, archive_path=>'t/data/sequence/100818_IL32_05174/Latest_Summary/archive', positions => [1], );

  #remove all the tags for the run, then the run will not be multiplex run
  $bam->npg_api_run->{tags} = [];

  is( scalar @{$bam->file_list()}, 2, "correct number of bam files for lane 1 if not multiplex run");

  $bam->clear_file_list(); 
  $bam->positions([1,4]);  
  is( scalar @{$bam->file_list()}, 3, "correct number of bam files for lane 1 and 4 if not multiplex run");
  
  $bam->clear_file_list();
  $bam->positions([]);
  is( scalar @{$bam->file_list()}, 6, "correct number of bam files for all lanes if not multiplex run");

  #reset run as multiplex run
  $bam->npg_api_run->{tags} = [qw(multiplex)];
  
  $bam->clear_file_list();
  $bam->positions([]);
  is( scalar @{$bam->file_list()}, 6, "correct number of bam files for all lanes");
  
  $bam->clear_file_list();
  $bam->positions([1, 2, 4]);
  is( scalar @{$bam->file_list()}, 6, "correct number of bam files for lane 1 2 and 4");
}

{
  my $bam = npg_common::irods::run::Bam->new(id_run => 5174, archive_path=>'t/data/sequence/100818_IL32_05174/Latest_Summary/archive', positions => [1], );
  my $bam_file = $bam->file_list()->[0];
  is($bam->get_number_of_reads_qc($bam_file), 2963516, 'correct total number reads for 5174 lane 1 from qc json');

  #$bam->get_number_of_reads_samtools($bam_file);

  is($bam->get_number_of_reads($bam_file), 2963516, 'correct total number reads for 5174 lane 1');

  is( $bam->number_of_reads_list()->{'5174_1'},  2963516, 'correct total number reads for 5174 lane 1 from cache');
  
  
}
1;
__END__
