#########
# Author:        Marina Gourtovaia
# Maintainer:    $Author: ajb $
# Created:       14 June 2010
# Last Modified: $Date: 2011-08-10 10:05:48 +0100 (Wed, 10 Aug 2011) $
# Id:            $Id: 10-request.t 13877 2011-08-10 09:05:48Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/useful_modules/tags/release-38.0/t/10-request.t $
#


use strict;
use warnings;
use Test::More tests => 34;
use Test::Exception;
use Test::Deep;
use Test::Warn;
use File::Temp qw/ tempdir /;
use File::Spec::Functions qw(catfile);
use File::Path qw(make_path );
use HTTP::Request::Common;
use Test::MockObject;

use st::api::batch;
use npg::api::run;
use st::api::util;
use st::api::asset;
use npg::api::util;

use t::useragent;

use_ok('npg_common::request');
my $VAR_NAME = q[];
my $TEST_CACHE = q[t/data/sequence];

my $supports_cache = 0;
eval {
  npg::api::util->new()->save2cache(1);
  $supports_cache = 1;
};

{
  my $r = npg_common::request->new();
  isa_ok($r, 'npg_common::request');
  $VAR_NAME = $r->cache_dir_var_name;
}

{
  is (npg_common::request->cache_dir_var_name, q[NPG_WEBSERVICE_CACHE_DIR], 'class attr for var name');
  is (npg_common::request->new()->cache_dir_var_name, q[NPG_WEBSERVICE_CACHE_DIR], 'class attr for var name works for an instance as well');
}


{
  my $r = npg_common::request->new();
  throws_ok { $r->make() } qr/Uri is not defined/, 'error if no uri given';
  throws_ok { $r->make(q[]) } qr/Uri is not defined/, 'error if an empty uri given';
 
  local $ENV{$VAR_NAME} = q[/dome/non-existing];
  throws_ok { $r->make(q[dodo], q[GET]) } qr/Cache directory \/dome\/non-existing does not exist/, 'error when a cache directory does not exist';

  local $ENV{$VAR_NAME} = q[Changes];
  throws_ok { $r->make(q[dodo], q[GET]) } qr/is not a directory/, 'error when a cache directory is not a directory';
}


{
  local $ENV{$VAR_NAME} = q[t];
  my $r = npg_common::request->new();

  is( $r->_create_path(q{http://intweb.sanger.ac.uk/perl/prodsoft/npg/npg/run/1234.xml}), 
                       q{t/npg/run/1234.xml}, q{npg path created ok} );
  is( $r->_create_path(q{http://psd-production.internal.sanger.ac.uk:6600/batches/6935.xml}), 
                       q{t/st/batches/6935.xml}, q{st path created ok});
  is( $r->_create_path(q{http://http://news.bbc.co.uk/sport1/hi/football/world_cup_2010/matches/match_01}), 
     q{t/ext/news.bbc.co.uk/sport1/hi/football/world_cup_2010/matches/match_01}, q{external path created ok} );
}


{
  my $r = npg_common::request->new();
  my $req = GET q[dodo];
  $r->_personalise_request($req);
  is ($req->header(q[X-username]), getlogin, 'username set in the request header');
}


{
  SKIP: {
    if (!$supports_cache) {
      skip 'Cache check skipped, npg and st apis do not support caching', 7;
    }
    my $sample_id = 100;
    ok (!-e catfile($TEST_CACHE, q[st], q[samples], $sample_id . q[.xml]), 'test prerequisite OK');

    throws_ok { st::api::sample->new({id => $sample_id, max_retries => 1,})->name } qr/Giving up. Last error/, 'error when resource does not exists, 1 retry';
    throws_ok { st::api::sample->new({id => $sample_id, max_retries => 3, retry_delay => 1,})->name } qr/No mock data configured for/, 'error when resource does not exists, 3 retries';

    $sample_id = 9837;
    ok (-e catfile($TEST_CACHE, q[st], q[samples], $sample_id . q[.xml]), 'test prerequisite OK');
    is ( st::api::sample->new({id => $sample_id, max_retries => 1,})->name, q[Multiplex-phix], 'sample fetched,  name correct');

    my $dir = tempdir( CLEANUP => 1 );
    local $ENV{$VAR_NAME} = $dir;
    ok (!-e catfile($dir, q[st], q[samples], $sample_id . q[.xml]), 'test prerequisite OK');
    throws_ok { st::api::sample->new({id => $sample_id, max_retries => 1,})->name } qr/is not in the cache/, 'error when fetching from a cache a resource that is not there';
  }
}


{
  SKIP: {
    if (!$supports_cache) {
      skip 'Cache check skipped, npg and st apis do not support caching', 2;
    }
    local $ENV{$VAR_NAME} = q[t/data/request];
    my $asset_id = 50801;
    ok (!-e catfile($TEST_CACHE, q[st], q[assets], $asset_id . q[.xml]), 'test prerequisite OK');
    is (st::api::asset->new({id => $asset_id, max_retries => 1,})->name, q[New GGO],'asset name attribute correct');
  }
}


{
  SKIP: {
    if (!$supports_cache) {
      skip 'Cache check skipped, npg and st apis do not support caching', 7;
    }

    my $batch_id = 4837;
    ok (-e catfile($TEST_CACHE, q[st], q[batches], $batch_id . q[.xml]), 'test prerequisite OK');

    my $batch1 = st::api::batch->new({id => $batch_id,});

    my $dir = tempdir( CLEANUP => 1 );
    local $ENV{$VAR_NAME} = $dir;
    throws_ok { st::api::batch->new({id => $batch_id,})->lane_count } qr/is not in the cache/, 'error when a resource is not in cache';

    my $util = st::api::util->new({save2cache => 1,});
    my $batch2;
    lives_ok { $batch2 = st::api::batch->new({util => $util, id => $batch_id, max_retries => 1,})->lane_count } 'call to request and save the resource lives';
    ok (-e catfile($dir, q[st], q[batches], $batch_id . q[.xml]), 'batch xml saved to cache');
  
    #warning_like { st::api::batch->new({util => $util, id => $batch_id, max_retries => 1,})->lane_count } qr/serving from cache/, 'warning when asked to save the resource that is already in the cache';

    $batch_id = 5347;
    ok (-e catfile($TEST_CACHE, q[st], q[batches], $batch_id . q[.xml]), 'test prerequisite OK');
    lives_ok { $batch2 = st::api::batch->new({util => $util, id => $batch_id, max_retries => 1,})->lane_count } 'call to save a second batch lives';
    ok (-e catfile($dir, q[st], q[batches], $batch_id . q[.xml]), 'second batch xml saved to cache');
  }
}


{
  SKIP: {
    if (!$supports_cache) {
      skip 'Cache check skipped, npg and st apis do not support caching', 3;
    }
  my $batch_id = 4837;
  my $original = catfile($TEST_CACHE, q[st], q[batches], $batch_id . q[.xml]);
  ok (-e $original, 'test prerequisite OK'); 
  my $batch1 = st::api::batch->new({id => $batch_id,});
  
  my $cache = tempdir( CLEANUP => 1 );
  local $ENV{$VAR_NAME} = $cache;
  make_path  catfile($cache, q[st], q[batches]);
  my $copy = catfile($cache, q[st], q[batches], $batch_id . q[.xml]);
  `cp $original $copy`;
  ok (-e $copy, 'test prerequisite OK');
  my $batch2 = st::api::batch->new({id => $batch_id,});
  cmp_deeply ($batch1, $batch2, 'the same object returned from cache and test repository');
  }
}


{
  my $content = q[Run 2888 tagged];

  my $mockUA = Test::MockObject->new();
  $mockUA->fake_new(q{LWP::UserAgent});

  my $fake_response = HTTP::Response->new(200, '200 Ok', undef, "$content");
  $mockUA->set_always('request', $fake_response);
  $mockUA->set_always('timeout', 60);
  $mockUA->set_always('agent', q[npg_common::request]);
  $mockUA->set_always('env_proxy', q[]);
  my $returned;
  lives_ok {$returned = npg_common::request->new()->make(q[http://intweb.sanger.ac.uk/perl/prodsoft/npg/npg/run/4913], q[POST])}, 'post request lives';
  is ($returned, $content, 'correct response returned');

  local $ENV{$VAR_NAME} = q[t];
  throws_ok {npg_common::request->new()->make(q[http://intweb.sanger.ac.uk/perl/prodsoft/npg/npg/run/4913], q[POST])}  qr/POST requests cannot use cache:/, 'post request croaks if cache is set';
}

