#########
# Author:        gq1
# Maintainer:    $Author: ajb $
# Created:       2009-04-17
# Last Modified: $Date: 2011-08-10 10:05:48 +0100 (Wed, 10 Aug 2011) $
# Id:            $Id: 10-sequence-SAM_Header.t 13877 2011-08-10 09:05:48Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/useful_modules/tags/release-38.0/t/10-sequence-SAM_Header.t $
#

use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Test::More tests => 45;
use Test::Exception;

use_ok('npg_common::sequence::SAM_Header');

local $ENV{TEST_DIR} = q[t/data];
my $qc_store = npg_qc::autoqc::qc_store->new(use_db => q[no]);

{
  my $picard_converter_cmd = 'java -jar t/bin/aligners/picard/current/SamFormatConverter.jar VALIDATION_STRINGENCY=SILENT INPUT=/dev/stdin OUTPUT=4219_1.bam';
  my $sam = npg_common::sequence::SAM_Header->new(
               {
                 id_run    => 4219,
                 bwa_cmd   => 'bwa',
                 samtools_cmd  => 'samtools',
                 position  => 1,
                 picard_converter_cmd => $picard_converter_cmd,
               });
       
  isa_ok($sam, 'npg_common::sequence::SAM_Header', 'object test');
  is ($sam->_get_picard_version_from_cmd( $picard_converter_cmd ), '1.31', 'correct picard version number');
  is ($sam->picard()->{CL}, $picard_converter_cmd, 'correct picard cl');
  is ($sam->picard()->{VN}, '1.31', 'correct picard VN');
  is ($sam->picard()->{ID}, 'picard_samformatconverter', 'correct picard ID');
  is ($sam->picard()->{PN}, 'Picard SamFormatConverter', 'correct picard name');
  
  is( $sam->picard_markduplicates()->{PN}, 'Picard MarkDuplicates', 'correct picard markdup0licates program name');
  is( $sam->picard_markduplicates()->{ID}, 'picard_markduplicates', 'correct picard markdup0licates program ID');
  
  is( scalar keys %{$sam->sam_original_quality}, 0, 'no sam_original_quality in pg list');
  my $sam_original_quality_cl = q{/software/solexa/bin/sam_original_quality.pl --qseq_dir dir --lane 1 --read_number_1 1};
  $sam->clear_sam_original_quality();
  $sam->sam_original_quality_cmd($sam_original_quality_cl);
  is( $sam->sam_original_quality()->{PN}, 'sam_original_quality', 'correct sam_original_quality PN');
  is( $sam->sam_original_quality()->{CL}, $sam_original_quality_cl, 'correct sam_original_quality CL');
  is( $sam->samtools_fixmate()->{ID}, 'samtools_fixmate', 'correct id for samtools fixmate');
}

my $cache = q[t/data/sequence];
my $cache_var_name = q[NPG_WEBSERVICE_CACHE_DIR];

local $ENV{$cache_var_name} = $cache;

{
  my $sam = npg_common::sequence::SAM_Header->new(
               {
                 id_run    => 4254,
                 position  => 1,
                 qc_store  => $qc_store,
                 picard_dict => 't/data/sequence/references/Streptococcus_pneumoniae/ATCC_700669/all/picard/S_pneumoniae_700669.fasta.dict'
               });
  is($sam->_construct_rg(1),'@RG	ID:1	PL:ILLUMINA	CN:SC	SM:87540	LB:87540	DT:2010-01-09	PU:100109_IL30_4254_1'
        .'	DS:Study ERP000139: http://www.sanger.ac.uk/resources/downloads/bacteria/streptococcus-pneumoniae', 'correct RG');
  is( $sam->_parsing_picard_dict(), 1, 'only one sequence in the reference dictionary');
}

{
  my $sam = npg_common::sequence::SAM_Header->new(
               {
                 id_run    => 4254,
                 position  => 1,
                 tag_index => 0,
                 qc_store  => $qc_store,
               });
  is($sam->_construct_rg(1),'@RG	ID:1	PL:ILLUMINA	CN:SC	SM:87540	LB:87540	DT:2010-01-09	PU:100109_IL30_4254_1#0'
  .'	DS:Study ERP000139: http://www.sanger.ac.uk/resources/downloads/bacteria/streptococcus-pneumoniae', 'correct RG');
}

{
  my $sam = npg_common::sequence::SAM_Header->new(
               {
                 id_run    => 4254,
                 position  => 1,
                 tag_index => 2,
                 human_split => q{human},
                 qc_store  => $qc_store,
               });
  is($sam->_construct_rg(1),'@RG	ID:1	PL:ILLUMINA	CN:SC	SM:042935PMEN1	LB:64424	DT:2010-01-09	PU:100109_IL30_4254_1_human#2'
  .'	DS:Study ERP000139: http://www.sanger.ac.uk/resources/downloads/bacteria/streptococcus-pneumoniae', 'correct RG');
}

{
  my $sam = npg_common::sequence::SAM_Header->new(
               {
                 id_run    => 4254,
                 position  => 4,
                 qc_store  => $qc_store,
               });
  is($sam->_construct_rg(1),'@RG	ID:1	PL:ILLUMINA	CN:SC	SM:79578	LB:79578	DT:2010-01-09	PU:100109_IL30_4254_4', 'correct RG');
}

{
  my $sam = npg_common::sequence::SAM_Header->new(
               {
                 id_run    => 4174,
                 position  => 8,
                 human_split => q{nonhuman},
                 qc_store  => $qc_store,
               });
  is($sam->_construct_rg(1),'@RG	ID:1	PL:ILLUMINA	CN:SC	SM:PD3965a	LB:86093	PU:4174_8_nonhuman	DS:Study Acute Lymphoblastic Leukemia Sequencing: Agilent whole exome hybridisation capture will be performed on genomic DNA derived from 50 Acute Lymphoblastic Leukemia samples and matched normal DNA from the same patients. Three lanes of Illumina GA sequencing will be performed on the resulting 100 exome libraries and mapped build 37 of the human reference genome to facilitate the identification of novel cancer genes. In addition, 500bp, NO_PCR total genomic libraries will be prepared from the same samples and five lanes of Illumina GA sequencing will be analysed to characterise genome wide, somatically acquired structural variation.', 'correct RG');
}

{
  my $sam = npg_common::sequence::SAM_Header->new(
               {
                 id_run    => 5169,
                 position  => 8,
                 qc_store  => $qc_store,
                 archive_path => 't/data/staging/IL19/outgoing/100817_IL19_05169/Data/Intensities/Bustard1.8.1a1_24-08-2010_RTA/GERALD_24-08-2010_RTA/archive/',
               });
 is( $sam->median_insert_size(), 90, 'run 5169 lane 8 median insert size');        
}

{
  my $sam = npg_common::sequence::SAM_Header->new(
               {
                 id_run    => 5169,
                 position  => 8,
                 tag_index => 9,
                 qc_store  => $qc_store,
                 archive_path => 't/data/staging/IL19/outgoing/100817_IL19_05169/Data/Intensities/Bustard1.8.1a1_24-08-2010_RTA/GERALD_24-08-2010_RTA/archive/',
               });
  is( $sam->median_insert_size(), 0, 'run 5169 lane 8 tag 9 no median insert size');        
        
}

{
  my $sam = npg_common::sequence::SAM_Header->new(
               {
                 id_run    => 5169,
                 position  => 8,
                 tag_index => 0,
                 qc_store  => $qc_store,
                 archive_path => 't/data/staging/IL19/outgoing/100817_IL19_05169/Data/Intensities/Bustard1.8.1a1_24-08-2010_RTA/GERALD_24-08-2010_RTA/archive/',
               });
 is( $sam->median_insert_size(), 90, 'run 5169 lane 8 tag 0 median insert size');      
}

{
  my $sam = npg_common::sequence::SAM_Header->new(
               {
                 id_run    => 5169,
                 position  => 8,
                 tag_index => 10,
                 qc_store  => $qc_store,
                 archive_path => 't/data/staging/IL19/outgoing/100817_IL19_05169/Data/Intensities/Bustard1.8.1a1_24-08-2010_RTA/GERALD_24-08-2010_RTA/archive/',
               });
  is( $sam->median_insert_size(), 90, 'run 5169 lane 8 tag 10 median insert size');   
}

{
  my $sam = npg_common::sequence::SAM_Header->new(
               { 
                  id_run            => 5169,
                  position          => 8,
                  recalibrated_path => 't/data/no_cal',
               });
  is( scalar keys %{$sam->recalibration()}, 0, 'no_cal for recalibrated_path') ;
}

{
  my $sam = npg_common::sequence::SAM_Header->new(
               { 
                  id_run            => 5281,
                  position          => 1,
                  archive_path       => 't/data/staging/ILorHSany_sf20/incoming/100914_HS3_05281_A_205MBABXX/Data/Intensities/PB_basecalls_20101016-172254/no_cal/archive',
               });

  my $schema_info = $sam->schema_info(); 
  is(scalar keys %{ $schema_info }, 0, 'no schema information available');
  ok( !$schema_info->{bcl_to_qseq}->{ID}, 'no bcl_to_qseq avilable');
  ok( !$schema_info->{fastq_creator}->{ID}, 'no fastq_creator avilable');
  ok( !$schema_info->{recalibration}->{ID},  'no recalibration id');
}

{
  my $sam = npg_common::sequence::SAM_Header->new(
               { 
                  id_run            => 5281,
                  position          => 2,
                  archive_path       => 't/data/staging/ILorHSany_sf20/incoming/100914_HS3_05281_A_205MBABXX/Data/Intensities/PB_basecalls_20101016-172254/no_cal/archive',
               });

  my $schema_info = $sam->schema_info();
  is(scalar keys %{ $schema_info }, 43, 'no schema information available');
  
  is( $schema_info->{bcl_to_qseq}->{ID}, 'bcl_to_qseq', 'correct bcl_to_qseq id');
  is( $schema_info->{fastq_creator}->{ID}, 'fastq_creator', 'correct fastq_creator id');
  ok( ! $schema_info->{recalibration}->{ID}, 'no recalibration id');
}

{
  my $sam = npg_common::sequence::SAM_Header->new(
               { 
                  id_run            => 5281,
                  position          => 1,
                  archive_path       => 't/data/staging/ILorHSany_sf20/incoming/100914_HS3_05281_A_205MBABXX/Data/Intensities/PB_basecalls_20101017-172254/PB_cal/archive',
               });
  my $schema_info = $sam->schema_info();
  is(scalar keys %{ $schema_info }, 47, 'number of programs from schema info');

  is( $schema_info->{bcl_to_qseq}->{ID},'bcl_to_qseq', 'bcl_to_qseq avilable');

  is( $schema_info->{fastq_creator}->{ID}, 'fastq_creator', 'fastq_creator avilable');

  is( $schema_info->{recalibration}->{ID}, 'recalibration', 'recalibration program id');
  is( $schema_info->{recalibration}->{PN}, 'pb_predictor', 'recalibration program name');

  is( $schema_info->{recalibration_alignment}->{ID}, 'recalibration_alignment', 'recalibration_alignment program id');
  is( $schema_info->{recalibration_alignment}->{PN}, 'pb_align', 'recalibration_alignement program name');
  
  is( $schema_info->{recalibration_table_creation}->{ID}, 'recalibration_table_creation', 'recalibration table creation program id');
  is( $schema_info->{recalibration_table_creation}->{PN}, 'pb_calibration', 'recalibration table creation program name');
  
  is( $schema_info->{split_spiked_phix}->{CL}, '/software/solexa/bin/fastq_srf_split.pl -fastq1 archive/1234_1_1.fastq -fastq2 archive/1234_1_2.fastq -fork_alignment -id_run 1234 -position 1 -output_dir archive', 'split_spiked_phix cl');
  is( $schema_info->{split_spiked_phix}->{PN}, 'fastq_split', 'split_spiked_phix PN');

  is( $schema_info->{split_nonconsented_sequence}->{PN}, 'fastq_split', 'split_nonconsented_seqnence PN');
}
{
  my $sam = npg_common::sequence::SAM_Header->new(
               { 
                  id_run            => 5281,
                  position          => 1,
                  bustard_path       => 't/data/staging/IL19/outgoing/100817_IL19_05169/Data/Intensities/Bustard1.8.1a1_24-08-2010_RTA',
               });
  is( $sam->bustard()->{PN}, 'Bustard', 'correct bustard software name from bustard confif xml file');
}

1;
