#########
# Author:        Marina Gourtovaia
# Maintainer:    $Author: mg8 $
# Created:       27 January 2010
# Last Modified: $Date: 2010-01-15 16:17:37 +0000 (Fri, 15 Jan 2010) $
# Id:            $Id: 10-sequence-reference.t 7881 2010-01-15 16:17:37Z mg8 $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/useful_modules/branches/prerelease-17.0/t/10-sequence-reference.t $
#

use strict;
use warnings;
use Test::More tests => 91;
use Test::Exception;
use Test::Deep;
use Carp;
use English qw(-no_match_vars);

use_ok('npg_common::sequence::reference::base_count');

{
  my $bc = npg_common::sequence::reference::base_count->new(reference_path => q[t/data/simple_reference]);
  isa_ok($bc, 'npg_common::sequence::reference::base_count');
}

{

  throws_ok {npg_common::sequence::reference::base_count->new()}
             qr/is required/, 'error when path to ref is no tset in the constructor';

  throws_ok {npg_common::sequence::reference::base_count->new(
                                        reference_path => q[t/data/simple_reference],
                                        bin_size => -4
                                                             )} 
             qr/must be a positive integer/, 'error when bin_size attr value fails type constraint';
  throws_ok {npg_common::sequence::reference::base_count->new(
                                        reference_path => q[t/data/simple_reference],
                                        bin_size => 0
                                                             )} 
             qr/must be a positive integer/, 'error when bin_size attr value fails type constraint';
  throws_ok {npg_common::sequence::reference::base_count->new(
                                        reference_path => q[t/data/simple_reference],
                                        slide => -2
                                                             )} 
             qr/must be a positive integer/, 'error when slide attr value fails type constraint';
  throws_ok {npg_common::sequence::reference::base_count->new(
                                        reference_path => q[t/data/simple_reference],
                                        slide => 0
                                                             )} 
             qr/must be a positive integer/, 'error when slide attr value fails type constraint';

  throws_ok { npg_common::sequence::reference::base_count->new(reference_path => q[t/data/simple])->run() } 
             qr/does not exist or is not readable/, 'error when path to ref does not exist';

}


{
  is (npg_common::sequence::reference::base_count->default_bin_size(), 9_007_199_254_740_992, 'default_bin_size class method ok');

  my $expected = {
          '100' => 'D', '103' => 'G', '104' => 'H', '107' => 'K', '109' => 'M', '110' => 'N', 
          '114' => 'R', '115' => 'S', '116' => 'T', '118' => 'V', '119' => 'W', '120' => 'N', 
          '121' => 'Y', '46'  => '',  '65'  => 'A', '66'  => 'B', '67'  => 'C', '68'  => 'D', 
          '71'  => 'G', '72'  => 'H', '75'  => 'K', '77'  => 'M', '78'  => 'N', '82'  => 'R', 
          '83'  => 'S', '84'  => 'T', '86'  => 'V', '87'  => 'W', '88'  => 'N', '89'  => 'Y',
          '95'  => '',  '97'  => 'A', '98'  => 'B', '99'  => 'C',};

  cmp_deeply (npg_common::sequence::reference::base_count->translation_table(), $expected, 'translation table class method ok');

  is (join(q[ ], @{npg_common::sequence::reference::base_count->output_base_codes()}),
                     q[A C G T M R W S Y K V H D B N], 'output codes returned');
}


{
  my $bc;
  my $ref =  q[t/data/simple_reference];
  lives_ok {$bc = npg_common::sequence::reference::base_count->new(
                            reference_path => $ref)} 'constractor with a ref path only lives';
  is ($bc->reference_path, $ref, 'ref path set correctly');
  is ($bc->bin_size, npg_common::sequence::reference::base_count->default_bin_size, 'bin_size value is set correctly to a default value');
  is ($bc->slide, npg_common::sequence::reference::base_count->default_bin_size, 'slide value set correctly to a default value');
}


{
  my $bc;
  my $bin_size = 4;
  lives_ok {$bc = npg_common::sequence::reference::base_count->new(
                                        reference_path => q[t/data/simple_reference],
                                        bin_size => $bin_size
                                                             )}
          'constractor with a ref path and bin_size lives';
  is ($bc->bin_size, $bin_size, 'bin_size value is set correctly');
  is ($bc->slide, $bin_size, 'if slide attr value is not set, it is get set to bin_size value');
}


{
  my $bc;
  my $bin_size = 4;
  my $slide = 7;
  lives_ok {$bc = npg_common::sequence::reference::base_count->new(
                                        reference_path => q[t/data/simple_reference],
                                        bin_size => $bin_size,
                                        slide    => $slide
                                                             )}
          'constractor with a ref path, bin_size and slide lives';
  is ($bc->bin_size, $bin_size, 'bin_size value is set correctly');
  is ($bc->slide, $slide, 'slide value is set correctly');
}


{
  my $bc = npg_common::sequence::reference::base_count->new(
                                        reference_path => q[t/data/simple_reference],
                                        bin_size => 4,
                                        slide    => 7,
                                                             );
  my $sb;
  lives_ok {$sb = $bc->summary} 'summary call after creating an object lives';
  
  lives_ok {$bc->bins} 'bins call after creating an object lives';
  is (@{$bc->bins}, 0, 'bins array is emty at start');
}


{
  my $bc = npg_common::sequence::reference::base_count->new(
                                        reference_path => q[t/data/simple_reference],
                                        bin_size => 4,
                                        slide    => 7,
                                                           );
  
  my $expected = {counts => {},};
  cmp_deeply ($bc->_new_bin_map(), $expected, 'new bin map without start coord');
  
  my $coord = 4;
  $expected = {counts => {}, start_coord => $coord};
  cmp_deeply ($bc->_new_bin_map($coord), $expected, 'new bin map with the start coord');

  throws_ok {$bc->_new_bin_map(-8)} qr/Start coordinate should be non-negative/, 'error when new bin start coord arg is negative';
}


{
  my $bc = npg_common::sequence::reference::base_count->new(
                                        reference_path => q[t/data/simple_reference],
                                        bin_size => 4,
                                        slide    => 7,
                                                           );
  my @bin_list;
  my $seq_name = q[];
  lives_ok {$bc->_process_bin(\@bin_list, $seq_name)} 'processing an undefined bin list lives';
  is (@{$bc->bins}, 0, 'number of stored bins is not increased');

  $seq_name = q[cr2];  
  my $bin1 = {start_coord => 2, counts => {'A' => 1, 'C' => 2,},};
  push @bin_list, $bin1;
  my $bin2 = {start_coord => 4, counts => {'A' => 3, 'C' => 2,},};
  push @bin_list, $bin2;
  $bc->_process_bin(\@bin_list, $seq_name);
  is (@{$bc->bins}, 1, 'number of stored bins is incremented');
  
  $bin1->{sequence_name} = $seq_name;
  cmp_deeply ($bc->bins->[0], $bin1, 'correct bin stored');
  is (scalar @bin_list, 1, 'bin list size decreased by 1');
  cmp_deeply ($bin_list[0], $bin2, 'correct bin at the bottom of the list');

  my $bin3 = {start_coord => 5, counts => {'A' => 8, 'C' => 3,},};
  push @bin_list, $bin3;

  $bc->_process_bin(\@bin_list, $seq_name, 1);
  $bin2->{sequence_name} = $seq_name;
  cmp_deeply ($bc->bins->[1], $bin2, 'correct bin stored');
  is (scalar @bin_list, 0, 'bin list cleared');
}


{
  my $bc = npg_common::sequence::reference::base_count->new(
                                        reference_path => q[t/data/simple_reference],
                                        bin_size => 3,
                                                           );
  throws_ok {$bc->run()} qr/contains unexpected character/  , 'error when a bad char present';
}

{
  my $bc = npg_common::sequence::reference::base_count->new(
                                        reference_path => q[t/data/sequence/base_count/empty.ref],
                                        bin_size => 3,
                                                           );
  lives_ok {$bc->run()} 'runs on empty ref';
  is (scalar @{$bc->bins}, 0, 'bin array is empty');
  cmp_deeply ($bc->summary, {counts => {}, ref_length => 0,}, 'summary shows zero length and no chars');
}

{
  my $bc = npg_common::sequence::reference::base_count->new(
                                        reference_path => q[t/data/sequence/base_count/no_seq_name.ref],
                                        bin_size => 3,
                                                           );
  lives_ok {$bc->run()} 'run method OK for a sequence without header';
  is (scalar @{$bc->bins}, 6, '6 bins in array');

  #abctdaa
  #aCtgaGgan

  my $b1 = { sequence_name => q[], start_coord => 0, counts=>{A => 1, 'B' => 1, 'C' => 1,},};
  my $b2 = { sequence_name => q[], start_coord => 3, counts=>{A => 1, 'D' => 1, 'T' => 1,},};
  my $b3 = { sequence_name => q[], start_coord => 6, counts=>{A => 2, C => 1, },};
  my $b4 = { sequence_name => q[], start_coord => 9, counts=>{G => 1, 'T' => 1, 'A' => 1,},};
  my $b5 = { sequence_name => q[], start_coord => 12, counts=>{A => 1, 'G' => 2,},};
  my $b6 = { sequence_name => q[], start_coord => 15, counts=>{N => 1,}, };

  cmp_deeply ($bc->bins->[0], $b1, 'bin No 1 OK');
  cmp_deeply ($bc->bins->[1], $b2, 'bin No 2 OK');
  cmp_deeply ($bc->bins->[2], $b3, 'bin No 3 OK');
  cmp_deeply ($bc->bins->[3], $b4, 'bin No 4 OK');
  cmp_deeply ($bc->bins->[4], $b5, 'bin No 5 OK');
  cmp_deeply ($bc->bins->[5], $b6, 'bin No 6 OK');

  my $summary = {counts => {'A' => 6, 'B' => 1, 'C' => 2, 'T' => 2, 'D' => 1, 'G' => 3, 'N' => 1,}, ref_length => 16,};
  cmp_deeply ($bc->summary, $summary, 'summary is correct');
}


{
  my $bc = npg_common::sequence::reference::base_count->new(
                                        reference_path => q[t/data/sequence/base_count/no_seq_name.ref],
                                        bin_size => 3,
                                        slide    => 2,
                                                           );
  $bc->run();

  is (scalar @{$bc->bins}, 8, '8 bins in array');

  #abctdaa
  #aCtgaGgan

  my $b1 = { sequence_name => q[], start_coord => 0, counts=>{A => 1, 'B' => 1, 'C' => 1,},};
  my $b2 = { sequence_name => q[], start_coord => 2, counts=>{C => 1, 'D' => 1, 'T' => 1,},};
  my $b3 = { sequence_name => q[], start_coord => 4, counts=>{A => 2, D => 1, },};
  my $b4 = { sequence_name => q[], start_coord => 6, counts=>{A => 2, C => 1, },};
  my $b5 = { sequence_name => q[], start_coord => 8, counts=>{G => 1, 'T' => 1, 'C' => 1},};
  my $b6 = { sequence_name => q[], start_coord => 10, counts=>{A => 1, 'G' => 2,},};
  my $b7 = { sequence_name => q[], start_coord => 12, counts=>{A => 1, 'G' => 2,},};
  my $b8 = { sequence_name => q[], start_coord => 14, counts=>{N => 1, 'A' => 1,}, };

  cmp_deeply ($bc->bins->[0], $b1, 'sliding bin No 1 OK');
  cmp_deeply ($bc->bins->[1], $b2, 'sliding bin No 2 OK');
  cmp_deeply ($bc->bins->[2], $b3, 'sliding bin No 3 OK');
  cmp_deeply ($bc->bins->[3], $b4, 'sliging bin No 4 OK');
  cmp_deeply ($bc->bins->[4], $b5, 'sliding bin No 5 OK');
  cmp_deeply ($bc->bins->[5], $b6, 'sliding bin No 6 OK');
  cmp_deeply ($bc->bins->[6], $b7, 'sliding bin No 7 OK');
  cmp_deeply ($bc->bins->[7], $b8, 'sliding bin No 8 OK');

  my $summary = {counts => {'A' => 6, 'B' => 1, 'C' => 2, 'T' => 2, 'D' => 1, 'G' => 3, 'N' => 1,}, ref_length => 16,};
  cmp_deeply ($bc->summary, $summary, 'summary is correct');
}


{
  my $bc = npg_common::sequence::reference::base_count->new(
                                        reference_path => q[t/data/sequence/base_count/no_seq_name.ref],
                                        bin_size => 3,
                                        slide    => 4,
                                                           );
  $bc->run();

  is (scalar @{$bc->bins}, 4, '4 bins in array');

  #abctdaa
  #aCtgaGgan

  my $b1 = { sequence_name => q[], start_coord => 0, counts=>{A => 1, 'B' => 1, 'C' => 1,},};
  my $b2 = { sequence_name => q[], start_coord => 4, counts=>{A => 2, 'D' => 1, },};
  my $b3 = { sequence_name => q[], start_coord => 8, counts=>{C => 1, T => 1, 'G' => 1, },};
  my $b4 = { sequence_name => q[], start_coord => 12, counts=>{A => 1, G => 2, },};

  cmp_deeply ($bc->bins->[0], $b1, 'sliding bin No 1 OK');
  cmp_deeply ($bc->bins->[1], $b2, 'sliding bin No 2 OK');
  cmp_deeply ($bc->bins->[2], $b3, 'sliding bin No 3 OK');
  cmp_deeply ($bc->bins->[3], $b4, 'sliging bin No 4 OK');

  my $summary = {counts => {'A' => 6, 'B' => 1, 'C' => 2, 'T' => 2, 'D' => 1, 'G' => 3, 'N' => 1,}, ref_length => 16,};
  cmp_deeply ($bc->summary, $summary, 'summary is correct');
}


{
  my $bc = npg_common::sequence::reference::base_count->new(
                                        reference_path => q[t/data/sequence/base_count/no_seq_name.ref],
                                                           );
  $bc->run();

  is (scalar @{$bc->bins}, 1, '1 bin in array when no bin size specified');

  #abctdaa
  #aCtgaGgan

  my $b1 = { sequence_name => q[], start_coord => 0, counts=>{'A' => 6, 'B' => 1, 'C' => 2, 'T' => 2, 'D' => 1, 'G' => 3, 'N' => 1,},};

  cmp_deeply ($bc->bins->[0], $b1, 'sliding bin No 1 OK');

  my $summary = {counts => {'A' => 6, 'B' => 1, 'C' => 2, 'T' => 2, 'D' => 1, 'G' => 3, 'N' => 1,}, ref_length => 16,};
  cmp_deeply ($bc->summary, $summary, 'summary is correct');
}


{
#>chr1
#abctdaa
#aCtgaGgan
#>chr2
#xaadWS
#>chr3
#dartYX

  my $bc = npg_common::sequence::reference::base_count->new(
                                        reference_path => q[t/data/sequence/base_count/three_seq.ref],
                                        bin_size => 4,
                                        slide    => 1,
                                                           );
  $bc->run();

  is (scalar @{$bc->bins}, 19, '19 bins in array');

  my $b1 = { sequence_name => q[chr1], start_coord => 0, counts=>{A => 1, 'B' => 1, 'C' => 1, 'T' => 1,},};
  my $b2 = { sequence_name => q[chr2], start_coord => 0, counts=>{A => 2, 'D' => 1, 'N' => 1,},};
  my $b3 = { sequence_name => q[chr3], start_coord => 0, counts=>{A => 1, D => 1, 'R' => 1, 'T' => 1, },};
  my $b4 = { sequence_name => q[chr3], start_coord => 2, counts=>{R => 1, T => 1, Y => 1, N => 1,},};

  cmp_deeply ($bc->bins->[0], $b1, 'sliding bin No 1 OK');
  cmp_deeply ($bc->bins->[13], $b2, 'sliding bin No 13 OK');
  cmp_deeply ($bc->bins->[16], $b3, 'sliding bin No 16 OK');
  cmp_deeply ($bc->bins->[18], $b4, 'sliging bin No 18 OK');

  my $summary = {counts => {'A' => 9, 'B' => 1, 'C' => 2, 'T' => 3, 'D' => 3, 'G' => 3, 'N' => 3, 'W' => 1, 'S' => 1, 'R' => 1, 'Y' => 1,}, ref_length => 28,};
  cmp_deeply ($bc->summary, $summary, 'summary is correct');
}


{
#>chr1
#_abc__tdaa
#.aCtgaG_ganga_

  my $bc = npg_common::sequence::reference::base_count->new(
                                        reference_path => q[t/data/sequence/base_count/with_gaps.ref],
                                        bin_size => 4,
                                                           );
  $bc->run();

  is (scalar @{$bc->bins}, 6, '6 bins in array');

  my $b1 = { sequence_name => q[chr1], start_coord => 1, counts=>{A => 1, 'B' => 1, 'C' => 1,},};
  my $b2 = { sequence_name => q[chr1], start_coord => 6, counts=>{A => 2, 'D' => 1, 'T' => 1,},};
  my $b3 = { sequence_name => q[chr1], start_coord => 11, counts=>{A => 1, G => 1, 'C' => 1, 'T' => 1, },};
  my $b4 = { sequence_name => q[chr1], start_coord => 15, counts=>{A => 1, G => 1,},};
  my $b5 = { sequence_name => q[chr1], start_coord => 18, counts=>{A => 1, G => 2, N => 1,},};
  my $b6 = { sequence_name => q[chr1], start_coord => 22, counts=>{A => 1, },};

  cmp_deeply ($bc->bins->[0], $b1, 'sliding bin No 1 OK');
  cmp_deeply ($bc->bins->[1], $b2, 'sliding bin No 2 OK');
  cmp_deeply ($bc->bins->[2], $b3, 'sliding bin No 3 OK');
  cmp_deeply ($bc->bins->[3], $b4, 'sliging bin No 4 OK');
  cmp_deeply ($bc->bins->[4], $b5, 'sliging bin No 5 OK');
  cmp_deeply ($bc->bins->[5], $b6, 'sliging bin No 6 OK');

  my $summary = {counts => {'A' => 7, 'B' => 1, 'C' => 2, 'T' => 2, 'D' => 1, 'G' => 4, 'N' => 1,}, ref_length => 24,};
  cmp_deeply ($bc->summary, $summary, 'summary is correct');
}


{
  my $bc = npg_common::sequence::reference::base_count->new(
                                        reference_path => q[t/data/sequence/base_count/three_seq.ref],
                                        bin_size => 4,
                                        slide    => 1,
                                                           );
  $bc->run();
  is($bc->actual_bin_size($bc->summary), 28, 'actual bin size returned correctly');
}


{
  my $bc = npg_common::sequence::reference::base_count->new(
                                        reference_path => q[t/data/sequence/base_count/three_seq.ref],
                                        bin_size => 4,
                                        slide    => 1,
                                                           );
  $bc->run();
  my $frozen;
  lives_ok {$frozen = $bc->freeze()} 'freezing the object to json lives';
  my $bc2 = npg_common::sequence::reference::base_count->thaw($frozen);
  cmp_deeply ($bc, $bc2, 'object from a frozen json string lokks like the original object');
}


{
  my $bc = npg_common::sequence::reference::base_count->new(
                                        reference_path => q[t/data/sequence/base_count/short_human.ref],
                                                           );
  $bc->run();

  is($bc->bins->[0]->{sequence_name}, q[3], 'first sequence name');
  is($bc->bins->[0]->{codes}->{C}, $bc->summary->{codes}->{C}, 'C count for summary and the first bin is the same');
  is($bc->bins->[0]->{codes}->{C}, $bc->summary->{codes}->{G}, 'G count for summary and the first bin is the same');

  my ($min_gc_summary, $max_gc_summary) = $bc->gc_content4bin($bc->summary);
  my ($min_gc_bin1, $max_gc_bin1)       = $bc->gc_content4bin($bc->bins->[0]);

  is ($min_gc_summary, $min_gc_bin1, 'min gc content the same from the bin and the summary');
  is ($max_gc_summary, $max_gc_bin1, 'max gc content the same from the bin and the summary');
  is ($min_gc_summary, 30/120,      'min gc content');
  is ($max_gc_summary, (30+18)/120, 'max gc content');
}
