#########
# Author:        Marina Gourtovaia
# Maintainer:    $Author: ajb $
# Created:       June 2010
# Last Modified: $Date: 2011-08-10 10:05:48 +0100 (Wed, 10 Aug 2011) $
# Id:            $Id: 10-sequence-reference-list.t 13877 2011-08-10 09:05:48Z ajb $
# $HeadURL: svn+ssh://svn.internal.sanger.ac.uk/repos/svn/new-pipeline-dev/useful_modules/tags/release-38.0/t/10-sequence-reference-list.t $
#

package reference;

use strict;
use warnings;
use Test::More tests => 15;
use Test::Exception;
use File::Spec::Functions qw(catfile);
use Cwd qw(cwd);
use Moose::Meta::Class;
use lib q{t};
use t::util;

use_ok('npg_common::sequence::reference::roles::list');

my $util = t::util->new();
{
  throws_ok { Moose::Meta::Class->create_anon_class(
         roles => [qw/npg_common::sequence::reference::roles::list/])
         ->new_object({ ref_repository => q[non-existing],}) }
     qr/Validation failed/, 'error message for non-existing repository';
}

SKIP: {
  my $dir = q[/lustre/scratch101/sanger/srpipe];
  skip 'No live reference repository', 2 unless -e $dir;
    
  my $live_lister =  Moose::Meta::Class->create_anon_class(
         roles => [qw/npg_common::sequence::reference::roles::list/])->new_object();
  is ($live_lister->ref_repository, join(q[/], $dir, q[references]), 'live reference repository location');
  is ($live_lister->adapter_repository, join(q[/], $dir, q[adapters]), 'live adapter repository location');
};

{
  my $repos = catfile(cwd, q[t/data/sequence]);
  my $lister = Moose::Meta::Class->create_anon_class(
          roles => [qw/npg_common::sequence::reference::roles::list/])
          ->new_object(repository => $repos,);
  throws_ok { $lister->repository_contents} qr/No default strain link for Clostridium_difficile/, 'repository listing error for no default';
}

{
  my $repos = catfile(cwd, q[t/data/sequence/references2]);
  my $lister = Moose::Meta::Class->create_anon_class(
          roles => [qw/npg_common::sequence::reference::roles::list/])
          ->new_object(repository=>q[t/data/sequence], ref_repository => $repos,);
  lives_ok { $lister->repository_contents} 'repository listing lives';
  my $full_report;
  lives_ok { $full_report = $lister->report } 'reporting lives';
  my $short_report;
  lives_ok { $short_report = $lister->short_report } 'short reporting lives';

  my @expexted_full = ('Species:Strain,Is Default?,Taxon Ids,Synonyms',
                       'Homo_sapiens:GRCh37_53,0,,',
                       'Homo_sapiens:NCBI36,1,1002;1003,Human;Other',
                       'Human_herpesvirus_4:EB1,1,,Epstein-Barr_virus',
                       'NPD_Chimera:010203,0,,',
                       'NPD_Chimera:010302,1,,',
                       'PhiX:Illumina,0,,',
                       'PhiX:Sanger,1,1007,'
                      );
  is ($full_report, join("\n", @expexted_full) . "\n", 'full report as expected');
  
  my @expected_short = ('Homo_sapiens (GRCh37_53)',
                        'Homo_sapiens (NCBI36)',
                        'Human_herpesvirus_4 (EB1)',
                        'NPD_Chimera (010203)',
                        'NPD_Chimera (010302)',
                        'PhiX (Illumina)',
                        'PhiX (Sanger)',
                        $npg_common::sequence::reference::roles::list::NO_ALIGNMENT_OPTION,
                       );
  is ($short_report, join("\n", @expected_short) . "\n", 'short report as expected');
}

{
  my $repos = catfile(cwd, q[t/data/sequence/references2]);
  my $lister = Moose::Meta::Class->create_anon_class(
          roles => [qw/npg_common::sequence::reference::roles::list/])
          ->new_object(ref_repository => $repos, all_species => 0, repository=>q[t/data/sequence],);
  lives_ok { $lister->repository_contents} 'repository listing lives';

  my @expexted_full = ('Species:Strain,Is Default?,Taxon Ids,Synonyms',
                       'Homo_sapiens:GRCh37_53,0,,',
                       'Homo_sapiens:NCBI36,1,1002;1003,Human;Other',
                       'Human_herpesvirus_4:EB1,1,,Epstein-Barr_virus',
                       'PhiX:Illumina,0,,',
                       'PhiX:Sanger,1,1007,'
                      );
  is ($lister->report, join("\n", @expexted_full) . "\n", 'full report as expected');
  
  my @expected_short = ('Homo_sapiens (GRCh37_53)',
                        'Homo_sapiens (NCBI36)',
                        'Human_herpesvirus_4 (EB1)',
                        'PhiX (Illumina)',
                        'PhiX (Sanger)',
                        $npg_common::sequence::reference::roles::list::NO_ALIGNMENT_OPTION,
                       );
  is ($lister->short_report, join("\n", @expected_short) . "\n", 'short report as expected');
  
  my $dir = $util->temp_directory();
  my $file =  catfile($dir, q[test]);
  lives_ok {$lister->report($file)} 'report with writing to file lives';
  ok((-e $file), 'file created');
}

1;