use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Test::More tests => 25;
use lib qw{t};
use t::util;
use DateTime;

use_ok('npg_common::mailer');

my $util = t::util->new({});
$util->catch_email();
{
  my $mailer = npg_common::mailer->new({
    to      => q{receive@tester.org},
    from    => q{send@tester.org},
    subject => q{A test subject},
    body    => qq{This is some test stuff\n\nLots of test stuff},
  });
  isa_ok($mailer, q{npg_common::mailer}, q{$mailer});
  eval { $mailer->mail(); };
  is($EVAL_ERROR, q{}, q{no croak mailing});
  my $mail = $util->parse_email();
  is($mail->{to}, qq{receive\@tester.org\n}, q{to is correct});
  is($mail->{from}, qq{send\@tester.org\n}, q{from is correct});
  is($mail->{subject}, qq{A test subject\n}, q{subject is correct});
  is($mail->{body}, qq{This is some test stuff\n\nLots of test stuff}, q{body is correct});
  is($mail->{precendence}, qq{list\n}, q{precendence is correct});
  is($mail->{cc}, undef, q{cc is correct});
  is($mail->{bcc}, undef, q{bcc is correct});
  is($mail->{content_type}, qq{text/plain\n}, q{content_type is correct});
}
{
  my $mailer;
  eval { $mailer = npg_common::mailer->new({
      from    => q{send@tester.org},
      subject => q{A test subject},
      body    => qq{This is some test stuff\n\nLots of test stuff},
    });
  };
  like($EVAL_ERROR, qr{Missing[ ]initializer[ ]label[ ]for[ ]npg_common::mailer:[ ]'to'\.}, q{croak as missing 'to' in args});
}
{
  my $mailer;
  eval { $mailer = npg_common::mailer->new({
      to      => q{receive@tester.org},
      subject => q{A test subject},
      body    => qq{This is some test stuff\n\nLots of test stuff},
    });
  };
  like($EVAL_ERROR, qr{Missing[ ]initializer[ ]label[ ]for[ ]npg_common::mailer:[ ]'from'\.}, q{croak as missing 'from' in args});
}
{
  my $mailer;
  eval { $mailer = npg_common::mailer->new({
      from    => q{send@tester.org},
      to      => q{receive@tester.org},
      body    => qq{This is some test stuff\n\nLots of test stuff},
    });
  };
  like($EVAL_ERROR, qr{Missing[ ]initializer[ ]label[ ]for[ ]npg_common::mailer:[ ]'subject'\.}, q{croak as missing 'subject' in args});
}
{
  my $mailer;
  eval { $mailer = npg_common::mailer->new({
      from    => q{send@tester.org},
      to      => q{receive@tester.org},
      subject => q{A test subject},
    });
  };
  like($EVAL_ERROR, qr{Missing[ ]initializer[ ]label[ ]for[ ]npg_common::mailer:[ ]'body'\.}, q{croak as missing 'body' in args});
}
{
  my $mailer = npg_common::mailer->new({
    to      =>  q{receive@tester.org second_recepient@tester.org},
    from    =>  q{send@tester.org},
    subject =>  q{A test subject},
    body    => qq{<xml>This is some test stuff\n\nLots of test stuff</xml>},
    bcc     =>  q{one_bcc@tester.org two_bcc@tester.org},
    cc      =>  q{one_cc@tester.org two_cc@tester.org},
    precedence => q{auto},
    type => q{text/xml},
  });
  isa_ok($mailer, q{npg_common::mailer}, q{$mailer (including optional headers)});
  eval { $mailer->mail(); };
  is($EVAL_ERROR, q{}, q{no croak mailing});
  my $mail = $util->parse_email();
  is($mail->{to}, qq{receive\@tester.org, second_recepient\@tester.org\n}, q{to is correct});
  is($mail->{from}, qq{send\@tester.org\n}, q{from is correct});
  is($mail->{subject}, qq{A test subject\n}, q{subject is correct});
  is($mail->{body}, qq{<xml>This is some test stuff\n\nLots of test stuff</xml>}, q{body is correct});
  is($mail->{precendence}, qq{auto\n}, q{precendence is correct});
  is($mail->{cc}, qq{one_cc\@tester.org, two_cc\@tester.org\n}, q{cc is correct});
  is($mail->{bcc}, qq{one_bcc\@tester.org, two_bcc\@tester.org\n}, q{bcc is correct});
  is($mail->{content_type}, qq{text/xml\n}, q{content_type is correct});
}
1;
