use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Test::More;
use lib qw{t};

if (!$ENV{MQ_TESTS}) {
  plan( skip_all => q{MQ_TESTS not set});
} else {
  plan( tests => 11 );
}

use_ok('npg_common::mq');
use_ok('npg_common::mq::stomp');

{
  my $mq = npg_common::mq->new();
  isa_ok($mq, q{npg_common::mq}, q{$mq});
}

{
  my $mq = npg_common::mq::stomp->new();
  isa_ok($mq, q{npg_common::mq::stomp}, q{$mq});
  ## fail to connect
  my $connection;
  eval { $connection = $mq->connection({
      hostname => q{foo},
      port => 10_000,
      client_id => q{spam},
    });
  };
  like($EVAL_ERROR, qr{Error[ ]connecting[ ]to[ ]foo:10000:}, q{unable to make a connection with given parameters});
}
{
  my $mq = npg_common::mq::stomp->new();

  ## test the util method
  $mq->util(q{spam});
  is($mq->util(), q{spam}, q{storage of user defined util ok});
  $mq->set_util(undef);
  my $util = $mq->util();
  isa_ok($util, q{srpipe::util}, q{$mq->util()});
  is($mq->util(), $util, q{util cache ok});

  ## test the connection method
  my $connection = $mq->connection();
  isa_ok($connection, q{Net::Stomp}, q{$mq->connection()});
  is($mq->connection(), $connection, q{connection cached ok, when recalled with no connection params});
  my $second_connection = $mq->connection({});
  isnt($second_connection, $connection, q{new connection made});
}


1;