use strict;
use warnings;
use Carp;
use English qw{-no_match_vars};
use Test::More 'no_plan';#tests => ;
use lib qw{t};
use IO::Prompt;

SKIP:
{
  eval {
    croak if !$ENV{MQ_TESTS};
  };
  skip (q{MQ_TESTS not set}, 1) if $EVAL_ERROR;
  
use npg_common::mq::stomp::topic;

{
  my $yes = prompt qq{During this test, you will be asked to confirm if you have killed or restarted your ActiveMQ server\n\nHave you started your server? };
  while (!($yes eq 'y' || $yes eq 'Y')) {
    $yes = prompt qq{Please confirm you have restarted your server: [y/Y]};
  }

  my $topic = npg_common::mq::stomp::topic->new();
  my $subscription;
  eval { $subscription = $topic->subscribe({
    topic => q{/topic/we/want/to/break/this},
    durable => q{true},
    client_id => q{ozzy_osbourne},
    ack => q{client},
  }); };
  is($EVAL_ERROR, q{}, q{1: no croak subscribing with a durable subscriber to selected topic});

  $yes = prompt qq{Please kill your server.\n\nConfirm: [y/Y]};
  while (!($yes eq 'y' || $yes eq 'Y')) {
    $yes = prompt qq{Please confirm you have killed your server: [y/Y]};
  }

  $yes = prompt qq{Please restart your server.\n\nConfirm: [y/Y]};
  while (!($yes eq 'y' || $yes eq 'Y')) {
    $yes = prompt qq{Please confirm you have restarted your server: [y/Y]};
  }

  eval { $subscription = $topic->subscribe({
    topic => q{/topic/we/want/to/break/this},
    durable => q{true},
    client_id => q{ozzy_osbourne},
    ack => q{client},
  }); };
  is($EVAL_ERROR, q{}, q{2: no croak subscribing with a durable subscriber to selected topic after killing server whilst still open subscription});

  $subscription = undef;
  $topic->set_connection(undef);
sleep 5;
  eval { $subscription = $topic->subscribe({
    topic => q{/topic/we/want/to/break/this},
    durable => q{true},
    client_id => q{ozzy_osbourne},
    ack => q{client},
  }); };
  is($EVAL_ERROR, q{}, q{3: no croak subscribing with a durable subscriber to selected topic after losing subscription});

  $topic->set_connection(undef);
sleep 5;
my $new_subscription;
  eval { $new_subscription = $topic->subscribe({
    topic => q{/topic/we/want/to/break/this},
    durable => q{true},
    client_id => q{ozzy_osbourne},
    ack => q{client},
    retries => 2,
  }); };
  is($EVAL_ERROR, q{}, q{4: no croak subscribing with a durable subscriber to selected topic after simulating run_away subscription});
  ok($topic->get_connection(), 'New connection made');

#  $topic->get_connection()->socket(undef);
#  $topic->get_connection()->select(undef);
sleep 5;
#  my $message;
#  eval { $message = $topic->get_message({
#    topic => q{/topic/we/want/to/break/this},
#    durable => q{true},
#    client_id => q{ozzy_osbourne},
#    ack => q{client},
#    retries => 2,
#  }); } or do { carp $EVAL_ERROR; };
#  is($EVAL_ERROR, q{}, q{5: no croak getting message with a durable subscriber to selected topic after simulating run_away subscription});

}
{
  my $topic = npg_common::mq::stomp::topic->new();
  my $arg_refs = {
    topic   => q{/topic/we/want/to/break/this},
    body => q{Black Sabbath},
    exchange => q{amq.topic},
    persistent => q{true},
  };
  eval { $topic->send($arg_refs); };
  is($EVAL_ERROR, q{}, q{no croak sending - topic used for destination});

  
}
{
  my $topic = npg_common::mq::stomp::topic->new();
  my $connection = $topic->connection({
    
  });
  sleep 1;
  my $yes = prompt qq{Please kill your server.\n\nConfirm: [y/Y]};
  while (!($yes eq 'y' || $yes eq 'Y')) {
    $yes = prompt qq{Please confirm you have killed your server: [y/Y]};
  }

  my $arg_refs = {
    topic   => q{/topic/we/want/to/break/this},
    body => q{Black Sabbath},
    exchange => q{amq.topic},
    persistent => q{true},
    connection => $connection,
  };
  eval { $topic->send($arg_refs); };
  like($EVAL_ERROR, qr{Error[ ]with[ ]socket[ ]connection}, q{croak sending - Error with socket connection});

  
}


} # end of total SKIP block
1;